@@ -32,7 +32,8 @@ module Google
       :normalize_unicode,
       :skip_serialization,
       :skip_deserialization,
-      :api_format_version)
+      :api_format_version,
+      :use_opencensus)
 
     # General client options
     class ClientOptions
@@ -73,6 +74,8 @@ module Google
       #   @return [Boolean] True if response should be returned in raw form instead of deserialized.
       # @!attribute [rw] api_format_version
       #   @return [Fixnum] Version of the error format to request/expect.
+      # @!attribute [rw] use_opencensus
+      #   @return [Boolean] Whether OpenCensus spans should be generated for requests. Default is true.
 
       # Get the default options
       # @return [Google::Apis::RequestOptions]
@@ -101,5 +104,6 @@ module Google
     RequestOptions.default.skip_serialization = false
     RequestOptions.default.skip_deserialization = false
     RequestOptions.default.api_format_version = nil
+    RequestOptions.default.use_opencensus = true
   end
 end
